package it.unipr.ce.dsg.deus.example.basic;

import java.util.ArrayList;
import java.util.Properties;

import it.unipr.ce.dsg.deus.automator.AutomatorLogger;
import it.unipr.ce.dsg.deus.automator.LoggerObject;
import it.unipr.ce.dsg.deus.core.Event;
import it.unipr.ce.dsg.deus.core.InvalidParamsException;
import it.unipr.ce.dsg.deus.core.Process;
import it.unipr.ce.dsg.deus.core.RunException;
import it.unipr.ce.dsg.deus.p2p.node.Peer;

public class LogDynamicsEvent extends Event {

	private static final String K_MAX = "kMax";
    int kMax = 10;
    
	public LogDynamicsEvent(String id, Properties params, Process parentProcess)
			throws InvalidParamsException {
		super(id, params, parentProcess);
		initialize();
	}

	public void initialize() throws InvalidParamsException {
		if (params.containsKey(K_MAX))
			kMax = Integer.parseInt(params.getProperty(K_MAX));
	}
	
	@Override
	public void run() throws RunException {
		AutomatorLogger a = new AutomatorLogger(engine.getLogFileName());
		ArrayList<LoggerObject> fileValue = new ArrayList<LoggerObject>();
		
		double kValues[] = new double[kMax+1];
		for (int i = 0; i < kValues.length; i++)
			kValues[i] = 0;
		
		Peer currentNode = null;
		for (int i = 1; i < this.engine.getNodes().size(); i++) {
			currentNode = (Peer) this.engine.getNodes().get(i);
			
			int k_n = currentNode.getNeighbors().size();
			//System.out.println("k_n = " + k_n);
			
			kValues[k_n]++;
		}
		
		int numPeers = this.engine.getNodes().size();
		fileValue.add(new LoggerObject("num-peers", numPeers));
		
		for (int i = 0; i < kValues.length; i++) 
			fileValue.add(new LoggerObject("" + i + "", (double)kValues[i]/(double)numPeers));
		
		a.write(this.engine.getVirtualTime(), fileValue);
	}

}
