package it.unipr.ce.dsg.deus.example.basic;

import java.util.Properties;
import java.util.Random;

import it.unipr.ce.dsg.deus.core.Event;
import it.unipr.ce.dsg.deus.core.InvalidParamsException;
import it.unipr.ce.dsg.deus.core.Process;
import it.unipr.ce.dsg.deus.core.RunException;
import it.unipr.ce.dsg.deus.p2p.node.Peer;

public class RandomWalkEvent extends Event {

	private float meanArrivalTriggeredWalk = 1;
	private int ttl = 10;
	
	public RandomWalkEvent(String id, Properties params, Process parentProcess)
			throws InvalidParamsException {
		super(id, params, parentProcess);
	}

	public Object clone() {
		RandomWalkEvent clone = (RandomWalkEvent) super.clone();
		clone.setTtl(3);		
		return clone;
	}
	
	public void setTtl(int newTtl) {
		this.ttl = newTtl;
	}
	
	@Override
	public void run() throws RunException {
			
		Random rng = this.engine.getSimulationRandom();
		
		// associate the event to a random peer
		Peer peer = (Peer) associatedNode;
		if (peer == null) 
			peer = (Peer) this.engine.getNodes()
			.get(rng.nextInt(this.engine.getNodes().size()));
		
		System.out.println("peer: " + peer.getKey());
		System.out.println("TTL: " + this.ttl);
		
		if (this.ttl == 0)
			return;
		
		// select a random neighbor
		Peer neighbor = peer.getNeighbors().
				get(rng.nextInt(peer.getNeighbors().size()));
		
		// set next event's triggering time
		float nextEventTriggeringTime = triggeringTime + meanArrivalTriggeredWalk;
		
		// create a new random walk event for the neighbor, with TTL = this.TTL-1
		RandomWalkEvent rwEvent = (RandomWalkEvent) this.getEngine().createEvent(RandomWalkEvent.class,
						nextEventTriggeringTime);
		rwEvent.setAssociatedNode(neighbor);
		rwEvent.setTtl(this.ttl - 1);
		
		// insert the event into the Engine's event list
		this.engine.insertIntoEventsList(rwEvent);

	}

}
